// -----------------------------------------------------------------------------------------------------
//
//     _____ __  ________    ____   ______     __ __ ___    ____  _________ 
//    / ___//  |/  / ___/   /  _/ _/_/ __ \   / //_//   |  / __ \/_  __/   |
//    \__ \/ /|_/ /\__ \    / / _/_// / / /  / ,<  / /| | / /_/ / / / / /| |
//   ___/ / /  / /___/ /  _/ /_/_/ / /_/ /  / /| |/ ___ |/ _, _/ / / / ___ |
//  /____/_/  /_//____/  /___/_/   \____/  /_/ |_/_/  |_/_/ |_| /_/ /_/  |_|
//                                                                        
//
//   MAIN.C
//
//   hlavni program sw GSM I/O karty, 8.5.2005
//   napsano pro uVision2 2.4a (Keil Software, Inc.)
//
//   Procesor: ATMEL T89C51CC01
//   Krystal: 11.0592MHz
//	 Memory model: SMALL
//
//   Soucasti programu: MT.C, STR.C, UFR.C; MT.H, STR.H, UFR.H
//   Pouzite knihovny: REG51CC01.H, stdio.h, string.h, ctype.h
//	 (castecne zavisle na paltforme x51 - funkce oznaceny x51)
//
// -----------------------------------------------------------------------------------------------------

#include <mt.h>
#include <str.h>
#include <ufr.h>

#include <REG51CC01.H>
#include <stdio.h>
#include <string.h>

// -----------------------------------------------------------------------------------------------------

#define _DEBUG										// zapnuti/vypnuti debug modu, pro release verzi vypnut!!!
#define TO_M 100									// nasobic timeoutu, standardne 100
#pragma nomoda2										// vypnuti multiple DPTR

// - KONFIGURACNI STRING PRO DEBUG A PROMENE PRO BODU BEHU ---------------------------------------------
#ifdef _DEBUG
  char code eeprom_cfg[] = "\X002\X002\X02F420605535895A420BC AAAAAAAAAAAAAAAAA\0\0";  
  int idata ms_stats = 0;
  int idata sec_stats = 0;							//doba provozu zarizeni od resetu
#endif

// - DEKLARACE GLOBALNICH PROMENYCH --------------------------------------------------------------------
char xdata mainbuffer[260];							//hlavni buffer pro prijmani dat od MT
char xdata config[70];								//konfiguracni buffer
char idata SMSCnr[15];								//pro ulozeni cisla SMS centra
unsigned char idata bufpos = 0;						//pozice "konce buferu"
int idata timeout;									//promena pro timeout seriove linky a citani zpozdeni
char idata sendNR[15];								//cislo na ktere se posilaji SMSky

// - DEKLARACE PRIKAZOVYCH KONSTANT	--------------------------------------------------------------------
char code on[]="ZAPNI ";
char code off[]="VYPNI ";
char code onall[]="ZAPNI VSE";
char code offall[]="VYPNI VSE";
char code info[]="INFO";


// - PREUSOVACI RUTINA VOLNE BEZICIHO TIMERU T0 (x51) ---------------------------------------------------
void timer0 (void) interrupt 1 
{
  timeout++;										//inkrementace promene timeout
  TL0 = 0xCC;										//znovunastaveni timeru
  TH0 = 0xD4;   
#ifdef _DEBUG										//pro zobrzani doby behu
  ms_stats++;
  if (ms_stats == 1000)
  {
    sec_stats++;
    ms_stats = 0; 
  }
#endif
}


// - PRERUSOVACI RUTINA OD SERIOVEHO PORTU (x51)---------------------------------------------------------
void serial (void) interrupt 4
{
  if (RI == 1)										//pokud je prijem (ignorovani TI - vysilani)
  { 
    if (SBUF!=0xA)									//ignorovani znaku \d
    {
      mainbuffer[bufpos] = SBUF;					//odlozeni prijateho znaku
      mainbuffer[bufpos+1]= 0;						//zakonceni buferu
      timeout = 0;									//smazani timeoutu
      bufpos++;										//inkrementace ukazatele na konec mainbuferu
    }
    RI = 0;											//zrus RI
  }
}


// - PASIVACE WATCHDOGU (x51 - nutnost implementace WDT) -----------------------------------------------
void rst_Watchdog(void)
{
  WDTPRG |= 0x07;
  WDTRST  = 0x1E;
  WDTRST  = 0xE1;
}


// - NASTAVENI SERIOVE LINKY ATD, PRECTENI KONFIGURACE (x51 - nutnost implementace EEPROM pameti) ------
void SerInt()
{
  char idata pom, znak;						 		//pomocna promena pro cteni z EEPROMky, pocitadlo...
  char xdata *point = 0;							//pro cteni z EEPROMky (ukazatel)

// - nacteni konfigurace z EEPROM ----------------------------------------------------------------------
  pom = 0;
  while (pom < 65)
  {
    EECON = 2; 										//mapovani EEPROM do XDATA
    znak = *point;									//precteni znaku
    EECON = 0;										//unmap
    config[pom] = znak;								//ulozeni znaku do configu
    point++;										//inc ukazatele do EEPROM
    pom++;
  }
  config[pom] = '\0';								//ukonceni bufferu

//pri debugu povolit define _DEBUG!!!
#ifdef _DEBUG
  strcpy(config,eeprom_cfg);
#endif

// - nastaveni seriove linky ---------------------------------------------------------------------------
  SCON  = 0x50;			           			 		//8b seriovy prenos, mod 1, povoleni prijmu 
  TMOD |= 0x20;				        		 		//casovac 1, mod 2, 8b se znovuplnenim
  PCON |= 0x80;										//double baude rate
  TH1   = 253;              			     		//19,2kB/s pri krysta lu 11,0592MHz
  TL1  = 253;
  TR1 = 1;                                   		//spusteni casovace 1
// - nastaveni volne beziciho timeru T0 ----------------------------------------------------------------
  TL0 = 0xCC; 		               			 		//naplneni hodnoty do volne beziciho timeru 0 (cca 1ms)
  TH0 = 0xD4;
  TR0 = 1;	 		          				 		//spusteni casovace 0
  ET0 = 1;			      					 		//povoleni preruseni od timeru 0 
  EA= 1;				    				 		//povoleni vsech preruseni
  copy(config, sendNR, 2, strpos(config, 'A')-2);	//ulozeni telefoniho cisla
  if ((config[3] < 30)||(config[3] > 60))			//pri nenelezeni konfigurace nepokracuj v cinosti
  {
    P3_6 = 0;
    while(1);
  }
// - 3x 100ms bliknuti signalizujici funkcnost hw procesoru --------------------------------------------
  for (pom = 0; pom < 6; pom++)
  {
    P3_6 =~P3_6;
    timeout = 0;
    while (timeout < TO_M);
  }
  P0 = config[1];									//nastaveni portu podle configu
}


// - CEKANI NA "PRETECENI" TIMEOUTU (x51) --------------------------------------------------------------
void WaitForRec(int to)
{
  ES = 1;											//zapnuti prijmu pres preruseni
  timeout = 0;										//nulovani timeoutu
  while (timeout < to){rst_Watchdog();};			//cekani + pasivace Wd
  ES = 0;											//vypnuti prijmu
}


// - SMAZANI BUFERU ------------------------------------------------------------------------------------
void ClearBuffer()
{
  timeout = 0;										//nulovani timeoutu
  bufpos = 0;										//nulovani pozice buferu
  mainbuffer[0] = '\0';								//vymazani prvniho znaku (buffer bude "prazdny")
}


// - POSLI TEXT PRES SERIOVOU LINKU A CEKEJ URCENOU DOBU (x51) ----------------------------------------
int SendAndWait(char *text, int Wait)
{
  ClearBuffer();									//smazani buferu
  TI = 1;
  printf(text);										//poslani textu pres seriovou linku
  TI = 0;
  WaitForRec(Wait);									//pockej zadanou dobu
  return(strlen(text)-1);							//vraceni delky posilaneho textu
}


// - TEST PRIPOJENI MT POMOCI ZASLANI PRIKAZU "AT" (x51) ----------------------------------------------
bit attest()
{
  bit idata ok = 0;									//pomocna promena
  TI = 1;
  printf("AT\r");									//odeslani testovaciho prikazu
  TI = 0; 
  ES = 1;
  while ((timeout < 6*TO_M)&&(!(ok = oktest(mainbuffer)))) {rst_Watchdog();};
  ES = 0;
  return ok;										//pokud je odpoved MT OK pak vrat 1
}


// - ODESLANI TEXTU SMS ZPRAVY (x51) -------------------------------------------------------------------
bit sendSMS(char *text, char *cislo)
{
  char xdata send[200];								//bufer pro pripravu SMS zpravy v PDU pro odeslani
  char xdata len[4];								//pro odlozeni delky
  char xdata SMSCnr_TMP[20];						//tmp pro odlozeni cisla sms centra
  char xdata Nr_TMP[20];						 	//tmp pro odlozeni cisla

  strcpy(SMSCnr_TMP,SMSCnr);						//odlozeni cisel	
  strcpy(Nr_TMP,cislo);
  ClearBuffer();									//smazani buffer
  rst_Watchdog();									//pasivace Wd
  encodeSMS(text, send, Nr_TMP, SMSCnr_TMP, len);	//vytvoreni zpravy v PDU
  strcpy(Nr_TMP,"AT+CMGS=");						//slozeni prikazu a velikosti dat
  strcat(Nr_TMP,len);
  strcat(Nr_TMP,"\n");
  SendAndWait(Nr_TMP, 20*TO_M);						//odeslani
  ClearBuffer(); 									//smazani bufferu
  strcat(send,"\X1A\X0D\0");						//zakonceni bufferu
  TI = 1;											
  ES = 1; 
  printf(send);										//odeslani SMSky v PDU
  timeout = 0; 										//pockani na odpoved OK/ERROR
  while ((timeout < 120*TO_M)&&(!oktest(mainbuffer))&&(!errtest(mainbuffer))){rst_Watchdog();};
  ES = 0;
  TI = 0;
  timeout = 0;
  if (timeout >= 120*TO_M) return 1;				//neprisla odpoved
  return 0;
}


// - POTVRZENI (x51) -----------------------------------------------------------------------------------
void confirm(char *nr)
{
  char xdata pombuffer[50];							//pomocny buffer
  if (config[2] == 2)								//nastaveno potvrzeni prozvonenim
  {
    WaitForRec(10*TO_M);							//chvilku pockame
	strcpy(pombuffer,"ATD");						//prikaz ATD (vytoceni cisla)
    if (nr[0] == '4') strcat(pombuffer,"+");		//pridani "+" do cisla (+420xxxxxx)
    strcat(pombuffer,nr);							//pridani cisla
    strcat(pombuffer,";\r");						//ukonceni
    SendAndWait(pombuffer,140*TO_M);				//poslani na MT (vytoceni a cekani 40*TO_M)
    SendAndWait("ATH\r",6*TO_M);					//zaveseni
  }
  if (config[2] == 3)								//nastaveno potvrzeni pomoci SMS
  {
    strcpy(pombuffer, "PROVEDENO!");				//slozeni textu SMS
    createSMS(pombuffer, 8, 1, P2, P0, 0);
	sendSMS(pombuffer, nr);							//odeslani SMS
  }
  ClearBuffer();									//smazani bufferu
}


// - DEKODOVANI A PROVEDENI PRICHOZI SMS (x51) ---------------------------------------------------------
void incSMS(char *smsPos)
{
  char idata pomch, pomch1;  						//pomocne promene pro urceni cisla vystupu
  char xdata pombuffer[50];							//pomocny buffer pro odeslani pozadavku na vypis SMS
  char xdata tmp[14];								//pomocny buffer
  char xdata cislo[14];								//pro ulozeni cisla prichozi SMS

  ClearBuffer();									//vymazani vstupniho bufferu
  strcpy(pombuffer, "AT+CMGR=");					//vytvoreni AT prikazu pro vypis SMS
  strcat(pombuffer, smsPos);
  strcat(pombuffer, "\r");
  SendAndWait(pombuffer,TO_M*25);					//cekani na prijem
  rst_Watchdog();									//pasivace Wd
  pomch = decodeSMS(mainbuffer,cislo);				//dekodovani SMS
  pomch1 = mainbuffer[strlen(mainbuffer)-1]-'0';	//ulozeni identifikatoru vystupu a prevod z ASCII na 1-9(bin)
  if ((pomch == 0)&&(accessNR(cislo, config)))      //pri uspesnem prevodu SMS a povolenem cisle
  {
    if ((pomch1 > 0)&&(pomch1<9))					//identifikator je v rozmezi 1 - 9
    {
      pomch = 1;  									//vytvoreni dat z idnetifikatoru vhodneho pro "maskovani portu"
      pomch = pomch << (8 - pomch1);

      if (find(mainbuffer, on)) P0 = P0 | pomch;	//zapnuti vystupu
      if (find(mainbuffer, off))					//vypnuti vystupu
      {
        pomch =~pomch;
        P0 = P0 & pomch;
      }
      confirm(cislo);								//potvrzeni prijeti
    }
    else											//pri neexistenci identifikatoru se jeste muze jednat o prikazy...
    {
      if (find(mainbuffer, onall)) 					//zapnuti vsech vystupu?
	  {
		P0 = 0xFF;									//nastav vystup
        confirm(cislo);								//potvrzeni prijeti
      }
	  if (find(mainbuffer, offall))					//vypnuti vsech vystupu?
	  {
        P0 = 0x00;									//nastav vystup
        confirm(cislo);								//potvrzeni prijeti
	  }
      if (find(mainbuffer, info))					//informacni SMS?
      {
        ClearBuffer();								//smazani bufferu
	    SendAndWait("AT+CSQ\r",15*TO_M);			//precteni velikosti signalu z MT
	    strcpy(pombuffer, "SIG: ");					//zapis text SIG:
        copy(mainbuffer, tmp, strpos(mainbuffer ,0x20)+1, strpos(mainbuffer ,',')-strpos(mainbuffer ,0x20)-1);
        strcat(pombuffer, tmp);						//zapis velikost signalu z odpovedi MT
        strcat(pombuffer, "; BAT: ");				//zapis text BAT:
        ClearBuffer();								//smaz buffer
        SendAndWait("AT+CBC\r",20*TO_M);			//velikost nabyti baterie MT
        copy(mainbuffer, tmp, strpos(mainbuffer ,',')+1, strpos(mainbuffer ,'O') - strpos(mainbuffer ,',')-3);
        strcat(pombuffer, tmp);						//uloz jeji velikost
        strcat(pombuffer, "%;");					//ukonci %
        createSMS(pombuffer, 8, 1, P2, P0, 0);		//pridej stav portu (parametry 8 a 1 nemaji vyznam)
        sendSMS(pombuffer, cislo);					//odesli SMS
      }
    }
    strcpy(pombuffer, "AT+CMGD=");					//sestaveni AT prikazu pro vymazani SMS
    strcat(pombuffer, smsPos);
    strcat(pombuffer, "\r"); 
#ifndef _DEBUG
    SendAndWait(pombuffer,TO_M*6);					//jeho odeslani a pockani na odpoved
#endif
  }
} 


// - ZPRACOVANI PROZVONENI (x51) -----------------------------------------------------------------------
void decodeRING(char *nr)
{ 
  if (accessNR(nr, config))							//prozvoneno z povoleneho cisla?
  {
    P3_7 = 0;										//aktivovani LED(zelene)
    if ((config[0] > 0) && (config[0] < 9))			//testovani platneho konfig. bitu
    {
      switch (config[0])							//nagace butu podle configu
      {
        case 8: P0_0 = ~P0_0; break;
        case 7: P0_1 = ~P0_1; break;
        case 6: P0_2 = ~P0_2; break;
        case 5: P0_3 = ~P0_3; break;
        case 4: P0_4 = ~P0_4; break;
        case 3: P0_5 = ~P0_5; break;
        case 2: P0_6 = ~P0_6; break;
        case 1: P0_7 = ~P0_7; break;
      }
    }
  }
  SendAndWait("ATH\r",8*TO_M);						//zaveseni...
}


// ----------------------------------------------------------------------------------------------------
//  HLAVNI PROGRAM (x51)
// ----------------------------------------------------------------------------------------------------
void main (void) 
{
  char idata smsnr[3];								//pro cteni cisla prijate SMS
  char idata contest;								//promena pro test pripojeni MT
  bit idata mainbreak; 								//break pri nedetekovani MT
  char idata P2_odkl, pocit, mask, pom1, pom2;		//dalsi pomocne promene
  char xdata tmp[200];								//pomocny buffer (pro pouziti ve funkcich)

  SerInt();											//inicializace
  rst_Watchdog();									//inicializace WatchDogu (2^21 - 1 clock cycles)
  P2_odkl = P2;										//odlozeni stavu portu P2

WDTRST = 1;

// - HLAVNI PROGRAMOVY CYKLUS (I.) --------------------------------------------------------------------
  while (1)											//hlavni "nekonecny cyklus"
  {		     
    P3_6 = 1;										//vypnuti LED(zelene) prace se SMS "enginem"
    if (P3_7 == 1) P3_7 = 0;						//blikani cervene ledky pri nepripojenem MT
    else P3_7 = 1;
	SendAndWait("ATE0\r",6*TO_M);					//odeslani testovaciho ATE0 (jeste vypnuti echa)
    if (oktest(mainbuffer))							//cekani na OK
    {
      P3_7 = 0;										//MT pripojen
      SendAndWait("AT+CLIP=1\r",6*TO_M);			//nastaveni zobrazeni zisla pri prozvoneni
      SendAndWait("AT+CNMI=1,1,0,2\r",6*TO_M);		//nastaveni natifikace pri prichodu SMS
      SendAndWait("AT+CSCA?\r",6*TO_M);				//vypis SMSC cisla
      if (readSMSC(mainbuffer, SMSCnr))				//platne SMSC cislo?
      { 
        pocit = -1;									//pocitadlo rotaci pro scan portu
		mainbreak = 0;								//promena pro break z cyklu pripojeni MT
		contest = 0;								//vynulovani promene pro test konektivity
        ClearBuffer();								//smazani bufferu
       	ES = 1;										//zapnuti prijmu

// - CYKLUS PRIPOJENEHO MT (II.) ----------------------------------------------------------------------
        while (!mainbreak)
        {
        rst_Watchdog(); 							//pasivace Wd
		if (timeout >= TO_M*3)						//pri 300 prazdnych ticich
          {
	        ES = 0;									//vypnuti prijmu
            P3_6 = 1;								//vypnuti LED(zelene)
// - TESTOVANI PROZVONENI (III.) ----------------------------------------------------------------------
            bufTestRING(mainbuffer, tmp, bufpos);	//tet "prozvoneni"
            if (strlen(tmp) > 5)					//pri pozitivnim telefonim cisle
            {
			  P3_6 = 0;								//aktivovani LED(zelene)
			  decodeRING(tmp);						//"dekoduj" prozvoneni
			  contest = 0;							//netreba testovat konektivitu
            }
			else									//neprozvoneno, treba prisla SMS...
            {
// - TESTOVANI NOVE SMS (IV.) -------------------------------------------------------------------------
			  bufTestSMS(mainbuffer, smsnr, bufpos);//test notifikace prichodu SMS
              if (smsnr[0] != 0) 					//test na predani cisla pametveho mista			
              {
                P3_6 = 0;							//aktivovani LED(zelene)
                incSMS(smsnr);						//provedeni prichozi SMS
			    contest = 0;						//prisla SMS netreba testovat pripojeni
              }
			}
            ClearBuffer();							//smazani bufferu

// - TEST PRIPOJENI K MT (V.) -------------------------------------------------------------------------
            if (contest == 3) 						//kazdych 3 test na pripojeni MT
            {
        	  mainbreak = ~attest();				//odeslani AT (test pripojeni MT)
              contest = 0;    						//nulovani contestu
            }
            ClearBuffer(); 							//smazani bufferu
            contest++;								//inc contestu
            ES = 1;									//zapnuti prijmu
          }

// - SCANOVANI ZMENY PORTU (VI.) -----------------------------------------------------------------------
          if (bufpos == 0)							//pokud je prazdny buffer
          {
            pocit++;								//pocitadlo pro maskovani rotaci
            if (pocit==8)pocit=0;					//nulovani pocitadla
            pom1=P2_odkl;							//odlozeni stavu odkladadla P2
            pom2=P2;								//odlozeni P2
            mask=1;									//prvni bit mask
            mask = mask << pocit;					//rotovani masku podle prave scanovaneho bitu P2
            pom1 = pom1 & mask;						//"nulovani" pom1 podle mask
            pom2 = pom2 & mask;						//"nulovani" pom2 podle mask
            if (pom1 != pom2)						//pri neschode nastala zmena na portu
            {
              ES = 0;								//vypnuti prijmu
              if (pom2 == 0)						//sepnuti vstupu cislo x
              {
                mask =~mask;						//negace mask
                P2_odkl = P2_odkl & mask;			//nulovani pomoci mask
                P3_6 = 0;							//aktivovani LED(zelene) prace se SMS "enginem" a odeslani SMS
                tmp[0] = '\0';						//nulovani bufferu
			    sendSMS(createSMS(tmp, pocit, 1, P2_odkl, P0, 1), sendNR);
              }
              else									//vypnuti vstupu cislo x
              {
                P2_odkl = P2_odkl | mask;			//nastaveni pomoci mask
                P3_6 = 0;							//aktivovani LED(zelene) prace se SMS "enginem"  a odeslani SMS
                tmp[0] = '\0';						//nulovani bufferu
                sendSMS(createSMS(tmp, pocit, 0, P2_odkl, P0, 1), sendNR);
              }
              ClearBuffer();  						//smazani bufferu
              ES = 1;								//zapnuti prijmu
            }
          }
        }       
      }   
	}
  }
}
